"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lexer_1 = require("../lexer/lexer");
const parser_1 = require("./parser");
const callback_1 = require("./callback");
/**
 * Parserを生成するためのファクトリクラス
 */
class ParserFactory {
    /**
     * 言語
     * @param {Language} language 解析する言語情報
     * @param {ParsingTable} parsing_table 構文解析表
     * @returns {Parser} 生成されたパーサ
     */
    static create(language, parsing_table) {
        const callback_controller = new callback_1.DefaultCallbackController(language);
        const lexer = new lexer_1.Lexer(language.lex);
        const parser = new parser_1.Parser(lexer, language.grammar, parsing_table);
        parser.setCallbackController(callback_controller);
        return parser;
    }
    static createAST(language, parsing_table) {
        const callback_controller = new callback_1.ASTConstructor(language);
        const lexer = new lexer_1.Lexer(language.lex);
        const parser = new parser_1.Parser(lexer, language.grammar, parsing_table);
        parser.setCallbackController(callback_controller);
        return parser;
    }
}
exports.ParserFactory = ParserFactory;
