import { GrammarDefinition } from "../def/language";
import { ParsingTable } from "../def/parsingtable";
import { ILexer } from "../lexer/lexer";
import { CallbackController } from "./callback";
/**
 * 構文解析器
 */
export declare class Parser {
    private lexer;
    private grammar;
    private parsingtable;
    private callback_controller;
    /**
     * @param {ILexer} lexer 字句解析の際に使用する字句解析器
     * @param {GrammarDefinition} grammar 解析する構文定義
     * @param {ParsingTable} parsingtable 解析する構文解析表
     */
    constructor(lexer: ILexer, grammar: GrammarDefinition, parsingtable: ParsingTable);
    /**
     * 使用するコールバックコントローラーを設定する
     * @param {CallbackController} cc 構文解析時に使用されるコントローラー
     */
    setCallbackController(cc: CallbackController): void;
    /**
     * 構文解析を実行する
     * @param {string} input 解析する入力文字列
     * @returns {any} 解析結果(返る結果はコントローラによって異なる)
     */
    parse(input: string): any;
    private _parse(inputs);
}
