import { Language, GrammarRule } from "../def/language";
import { Token } from "../def/token";
import { FirstSet } from "./firstset";
import { SymbolDiscriminator } from "./symboldiscriminator";
/**
 * 言語定義から得られる、構文規則に関する情報を管理するクラス
 */
export declare class GrammarDB {
    private grammar;
    private _start_symbol;
    private _first;
    private _symbols;
    private tokenmap;
    private tokenid_counter;
    private rulemap;
    constructor(language: Language);
    /**
     * それぞれの記号にidを割り振り、Token->numberの対応を生成
     */
    private initTokenMap();
    /**
     * ある記号を左辺とするような構文ルールとそのidの対応を生成
     */
    private initDefMap();
    /**
     * 開始記号を得る
     */
    readonly start_symbol: Token;
    /**
     * First集合を得る
     * @returns {FirstSet}
     */
    readonly first: FirstSet;
    /**
     * 終端/非終端記号分類器を得る
     * @returns {SymbolDiscriminator}
     */
    readonly symbols: SymbolDiscriminator;
    /**
     * 構文規則がいくつあるかを返す ただし-1番の規則は含めない
     */
    readonly rule_size: number;
    /**
     * 与えられたidの規則が存在するかどうかを調べる
     * @param {number} id
     * @returns {boolean}
     */
    hasRuleId(id: number): boolean;
    /**
     * 非終端記号xに対し、それが左辺として対応する定義を得る
     *
     * 対応する定義が存在しない場合は空の配列を返す
     * @param x
     */
    findRules(x: Token): Array<{
        id: number;
        rule: GrammarRule;
    }>;
    /**
     * 規則idに対応した規則を返す
     *
     * -1が与えられた時は S' -> S $の規則を返す
     * @param id
     */
    getRuleById(id: number): GrammarRule;
    /**
     * [[Token]]を与えると一意なidを返す
     * @param {Token} token
     * @returns {number}
     */
    getTokenId(token: Token): number;
}
