import { Language } from "../def/language";
import { ParsingTable } from "../def/parsingtable";
import { Parser } from "../parser/parser";
/**
 * 言語定義から構文解析表および構文解析器を生成するパーサジェネレータ
 */
export declare class ParserGenerator {
    private language;
    private parsing_table;
    private table_type;
    private grammardb;
    private dfa_generator;
    /**
     * @param {Language} language 言語定義
     */
    constructor(language: Language);
    /**
     * 構文解析表の生成
     */
    private init();
    /**
     * 構文解析器を得る
     * @returns {Parser}
     */
    getParser(): Parser;
    /**
     * 構文解析表を得る
     * @returns {ParsingTable}
     */
    getParsingTable(): ParsingTable;
    /**
     * 生成された構文解析表に衝突が発生しているかどうかを調べる
     * @returns {boolean}
     */
    isConflicted(): boolean;
    /**
     * 構文解析表の種類を得る
     *
     * パーサジェネレータはまずLALR(1)構文解析表を生成し、LALR(1)構文解析表にコンフリクトを検知した場合はLR(1)構文解析表を使用する
     * @returns {"LR1" | "LALR1" | "CONFLICTED"}
     */
    getTableType(): "LR1" | "LALR1" | "CONFLICTED";
    /**
     * DFAから構文解析表を構築する
     * @param {DFA} dfa
     */
    private generateParsingTable(dfa);
}
